<?php
/**
 * PinpointUpgrade
 *
 * @copyright Copyright © 2017 Pinpoint Designs. All rights reserved.
 * @author    Ben Unsworth
 */

namespace Pinpoint\Core\Model\Setup;

use Magento\Framework\Setup\UpgradeDataInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Module\Dir\Reader;
use Magento\Framework\Xml\Parser;

class PinpointUpgrade implements UpgradeDataInterface
{
    /**
     * @var \Magento\Framework\Module\Dir\Reader
     */
    protected $moduleDirReader;

    /**
     * @var \Magento\Framework\Xml\Parser
     */
    protected $parser;

    /**
     * PureUpgrade constructor.
     * @param \Magento\Framework\Module\Dir\Reader $moduleReader
     * @param \Magento\Framework\Xml\Parser $reader
     */
    public function __construct(
        Reader $moduleReader,
        Parser $reader
    ) {
        $this->moduleDirReader = $moduleReader;
        $this->parser = $reader;
    }

    /**
     * upgrade
     * {@inheritdoc}
     * @SuppressWarnings(PHPMD.NPathComplexity)
     */
    public function upgrade(
        ModuleDataSetupInterface $setup,
        ModuleContextInterface $context
    ) {
        $setup->startSetup();

        $_currentVersion = $this->_incrementVersionNumber($context->getVersion());
        $_latestVersion = $this->_getLatestVersion();

        while (version_compare($_currentVersion, $_latestVersion, '<=')) {
            $_upgradeMethod = sprintf('upgrade_%s', str_replace('.', '_', $_currentVersion));
            if (method_exists($this, $_upgradeMethod)) {
                $this->{$_upgradeMethod}($setup);
            }
            $_currentVersion = $this->_incrementVersionNumber($_currentVersion);
        }

        $setup->endSetup();
    }

    /**
     * getLatestVersion - Read from module module.xml file to get setup_version value
     * @return string
     */
    protected function _getLatestVersion()
    {
        $filePath = $this->moduleDirReader->getModuleDir('etc', $this->_getCurrentModuleName()) . DIRECTORY_SEPARATOR . 'module.xml';
        $parsedXML = $this->parser->load($filePath)->xmlToArray();

        return $parsedXML['config']['_value']['module']['_attribute']['setup_version'];
    }

    /**
     * _getCurrentModuleName - Returns module name in the format Namespace_Module
     * @return string
     */
    protected function _getCurrentModuleName()
    {
        $explode = explode("\\", get_class($this));

        return sprintf("%s_%s", $explode[0], $explode[1]);
    }

    /**
     * _incrementVersionNumber - Increments version number to next version
     * @param $version
     * @return string
     */
    protected function _incrementVersionNumber($version)
    {
        $parts = explode('.', $version);
        if(count($parts) == 3) {
            if ($parts[2] + 1 < 10) {
                $parts[2]++;
            } else {
                $parts[2] = 0;
                if ($parts[1] + 1 < 10) {
                    $parts[1]++;
                } else {
                    $parts[1] = 0;
                    $parts[0]++;
                }
            }
        } else {
            $parts = [0,0,1];
        }

        return implode('.', $parts);
    }
}